<?php
/**
 * The main class for initialising integrations.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 */

namespace Search_Filter;

use Search_Filter\Integrations\Gutenberg;
use Search_Filter\Integrations\Legacy;
use Search_Filter\Integrations\WooCommerce;
use Search_Filter\Integrations\Themes;
use Search_Filter\Integrations\Settings as Integrations_Settings;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Loads all 3rd party integrations
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/includes
 */
class Integrations {

	private static $settings_init = false;
	private static $enabled_integrations = array();

	/**
	 * @since    3.0.0
	 */
	public function __construct() {

	}
	public static function init() {
		// Register settings.
		add_action( 'init', array( __CLASS__, 'register_settings' ), 1 );
		
		// Run the integrations.
		self::init_integrations();
		do_action( 'search-filter/integrations/init' );

	}
	public static function register_settings() {
		// Register settings.
		Settings::register( 'integrations', Integrations_Settings::get_data(), Integrations_Settings::get_groups() );
	}
	public static function init_integrations() {
		Legacy::init();
		Gutenberg::init();
		WooCommerce::init();
		Themes::init();
	}
	public static function get_integrations() {
		$defaults            = Settings::get_section_defaults( 'integrations' );
		$integrations_option = Options::get_option_value( 'integrations' );
		$integrations        = $defaults;
		if ( $integrations_option ) {
			$integrations = wp_parse_args( json_decode( $integrations_option ), $defaults );
		}

		return $integrations;
	}
	public static function get_enabled_integrations() {
		if ( self::$settings_init ) {
			return self::$enabled_integrations;
		}
		$integrations         = self::get_integrations();
		$enabled_integrations = array();
		foreach ( $integrations as $integration => $enabled ) {
			if ( $enabled ) {
				$enabled_integrations[] = $integration;
			}
		}
		self::$enabled_integrations = $enabled_integrations;
		self::$settings_init = true;
		return self::$enabled_integrations;
	}
	public static function is_enabled( $integration ) {
		if ( self::$enabled_integrations ) {
			return in_array( $integration, self::$enabled_integrations );
		}
		$integrations = self::get_enabled_integrations();
		return in_array( $integration, $integrations );
	}
}
