<?php
/**
 * Looks for `search_filter_queries` in a WP_Query (pre_get_posts), and takes over the query
 * parses url args + query settings into queries to made on our own tables
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */
namespace Search_Filter;

use Search_Filter\Core\Data_Store;
use Search_Filter\Database\Queries\Options as Options_Query;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Options {
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	/*
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}*/
	private static function get_option( $name ) {
		$option = Data_Store::get( 'options', $name );

		if ( $option ) {
			return $option;
		}

		$query_args = array(
			'number' => 1, // Only retrieve a single record.
			'name'   => $name,
		);
		/**
		 * TODO - we probably want to wrap this in our settings API
		 * so we never call the same field twice (maybe we need to
		 * update the API to support searching for fields without
		 * query ID)
		 */
		$query = new Options_Query( $query_args );

		// Bail if nothing found.
		if ( empty( $query->items ) ) {
			// return new \WP_Error( 'search_filter_option_not_found', __( 'Option not found.', 'search-filter' ) );
			return false;
		}

		return $query->items[0];
	}
	public static function get_option_value( $name ) {
		$option = self::get_option( $name );
		if ( $option ) {
			return $option->get_value();
		}
		return false;
	}

	public static function update_option_value( $name, $value ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$query          = new Options_Query();
		$updated_option = array(
			'name'  => $name,
			'value' => $value,
		);

		$option = self::get_option( $name );

		if ( $option ) {
			$result = $query->update_item( $option->get_id(), $updated_option );
		} else {
			$result = $query->add_item( $updated_option );
		}
		return $result;
	}
}
