<?php
/**
 * Looks for `search_filter_queries` in a WP_Query (pre_get_posts), and takes over the query
 * parses url args + query settings into queries to made on our own tables
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */
namespace Search_Filter;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Query {
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 */
	public static function init() {
		// Try to be the last thing to attach to the hook.
		add_action( 'pre_get_posts', 'Search_Filter\\Query::setup_queries', 100, 1 );
	}

	/**
	 * Setup the queries.
	 *
	 * @param \WP_Query $query
	 * @return void
	 */
	public static function setup_queries( $query ) {

		if ( ! self::is_frontend_query() ) {
			return;
		}

		$query_args = array();

		$query_type = 'wp';

		if ( $query_type === 'wp' ) {
			$query_handler = new \Search_Filter\Query\Handler\Wp( $query );
			$query_args    = $query_handler->get_query_args();
		}

		// Now try to update the query from the provided args.
		foreach ( $query_args as $key => $value ) {
			$query->set( $key, $value );
		}
	}

	// Make sure we're only filtering frontend queries.
	public static function is_frontend_query() {
		if ( ! is_admin() || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			return true;
		}
	}
}
