<?php
/**
 * Field settings for use in blocks and admin pages
 * TODO - probably could just be json - we've removed dependencies on saved settings
 * and fetch things via the rest api instead.
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter/Settings
 */

namespace Search_Filter\Features;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class that contains the settings found for styles
 */
class Settings {
	/**
	 * Returns the settings groups (name + label)
	 *
	 * @return array
	 */
	public static function get_groups() {
		$groups_data = array();
		return $groups_data;
	}

	// TODO - this should be moved into a parent class and extended.
	public static function get_defaults() {
		$defaults = array();
		$settings = self::get_data();
		foreach ( $settings as $setting ) {
			$defaults[ $setting['name'] ] = isset( $setting['default'] ) ? $setting['default'] : '';
		}
		return $defaults;
	}
	/**
	 * Returns all the settings.
	 *
	 * @return array
	 */
	public static function get_data() {

		$settings_data = array(
			array(
				'name'        => 'lazyLoadJavaScript',
				'label'       => __( 'Lazy load JavaScript', 'search-filter' ),
				'description' => __( 'Enabling this option ensures that Search & Filter JavaScript files are loaded only when needed - speeding up the other pages on your site.  Not all themes support this.', 'search-filter' ),
				'default'     => false,
				'type'        => 'string',
				'inputType'   => 'SettingToggle',
				'link'        => 'https://searchandfilter.com/documentation/using-the-block-editor/',
				'icon'        => 'wordpress',
				'iconColor'   => '#0073aa',
			),
			array(
				'name'        => 'removeDataOnUninstall',
				'label'       => __( 'Remove data on uninstall', 'search-filter' ),
				'description' => __( 'Removes all Search & Filter data when uninstalling this plugin.  Disable this to to keep your data if you plan to re-install later.', 'search-filter' ),
				'default'     => false,
				'type'        => 'string',
				'inputType'   => 'SettingToggle',
				'link'        => 'https://searchandfilter.com/documentation/using-the-block-editor/',
				'icon'        => 'wordpress',
				'iconColor'   => '#0073aa',
			),
			array(
				'name'        => 'debugMode',
				'label'       => __( 'Enable debugging tools', 'search-filter' ),
				'description' => __( 'Enables the debugging admin bar menu, helping to provide additional information when troubleshooting issues.', 'search-filter' ),
				'default'     => true,
				'type'        => 'string',
				'inputType'   => 'SettingToggle',
				'link'        => 'https://searchandfilter.com/documentation/using-the-block-editor/',
				'icon'        => 'wordpress',
				'iconColor'   => '#0073aa',
			),
		);
		return $settings_data;
	}
}
