<?php
/**
 * Select Filter Class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter
 * @subpackage Search_Filter/Fields/Filter
 */

namespace Search_Filter\Fields\Filter;

use Search_Filter\Fields\Filter;

/**
 * Extends `Choice` class and add overrides to Generate a Select field
 */
class Select extends Filter {
	public $icons = array(
		'arrow-down',
		'clear',
	);
	/**
	 * Override the default attributes
	 *
	 * @param array $defaults The default field attributes.
	 * @return array The new defaults
	 */
	public function get_default_attributes( $defaults = array() ) {
		$defaults                = parent::get_default_attributes();
		$defaults['placeholder'] = '';
		$defaults['multiple']    = 'no';
		return wp_parse_args( $defaults, $defaults );
	}
	/**
	 * Override the init and setup render data + escaping functions.
	 *
	 * @since    3.0.0
	 */
	public function init() {
		parent::init();

		// Defaults.
		$render_data = array(
			'options'         => array(),
			'statusText'      => '',
			'selectionLabel'  => '',
			'multiple'        => false,
			'selection'       => array(),
			// 'scale'            => $this->attributes['scale'],
			'placeholderText' => $this->attributes['placeholder'],
		);

		// Init multiple.
		$multiple                = $this->attributes['multiple'] === 'yes' ? true : false;
		$render_data['multiple'] = $multiple;

		// Set options.
		// TODO - since we cant't display the dropdown until JS is loaded, is this even necessary?
		$options                = $this->get_options();
		$render_data['options'] = $options;

		$selected_options = array();
		if ( $multiple ) {
			foreach ( $options as $option ) {
				if ( in_array( $option['value'], $this->get_values(), true ) ) {
					$selected_options[] = $option;
				}
			}
			$render_data['selection'] = $selected_options;
			if ( count( $selected_options ) > 0 ) {
				$render_data['placeholderText'] = '';
			}
		} else {
			foreach ( $options as $option ) {
				if ( in_array( $option['value'], $this->get_values(), true ) ) {
					$render_data['selectionLabel'] = $option['label'];
				}
			}
			if ( $render_data['selectionLabel'] !== '' ) {
				$render_data['placeholderText'] = '';
			}
		}

		$this->set_render_data( $render_data );

		$esc_callbacks = array(
			'options'         => array(
				'label' => 'esc_html',
				'value' => 'esc_attr',
				'depth' => 'intval',
			),
			'placeholderText' => 'esc_attr',
			'statusText'      => 'esc_html',
			'selectionLabel'  => 'esc_html',
			'multiple'        => 'boolval',
			'selection'       => array(
				'label' => 'esc_html',
				'value' => 'esc_attr',
			),
			'scale'           => 'sanitize_key',
		);
		$this->set_render_escape_callbacks( $esc_callbacks );
	}
}
