<?php
namespace Search_Filter\Integrations\Gutenberg;

use Search_Filter\Settings;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Parses Gutenberg blocks using regex
 */
class Block_Parser {

	public static function extract_blocks( $content, $return = 'raw' ) {

		$block_types = array(
			'search',
			'filter',
			'control',
			'reusable-field',
		);
		$blocks      = array();
		foreach ( $block_types as $block_type ) {
			if ( $return === 'raw' ) {
				$blocks = array_merge( $blocks, self::extract_blocks_by_type( $content, $block_type ) );
			} elseif ( $return === 'array' ) {
				$blocks = array_merge( $blocks, self::extract_blocks_by_type( $content, $block_type, '\Search_Filter\Integrations\Gutenberg\Block_Parser::parse_block' ) );
			} elseif ( $return === 'field' ) {
				$blocks = array_merge( $blocks, self::extract_blocks_by_type( $content, $block_type, '\Search_Filter\Integrations\Gutenberg\Block_Parser::parse_block_with_defaults' ) );
			}
		}
		// TODO - we need to grab any re-usable fields that are connected to a "dynamic" query

		return $blocks;
	}
	private static function escape_pattern_slashes( $str ) {
		return str_replace( '/', '\/', $str );
	}
	public static function extract_blocks_by_type( $content, $type, $map_callback = null ) {
		$blocks = array();

		// All blocks
		// $block_regex = '/<!-- wp:search-filter\/(.+?)-->/';

		// Specific block
		$tag_start = '<!-- wp:search-filter/' . $type . ' ';
		$tag_end   = ' /-->';
		$pattern   = '/' . self::escape_pattern_slashes( $tag_start ) . '(.*?)' . self::escape_pattern_slashes( $tag_end ) . '/';

		preg_match_all( $pattern, $content, $matches, PREG_SET_ORDER );

		if ( ! empty( $matches ) ) {
			foreach ( $matches as $match ) {
				if ( $map_callback ) {
					$blocks[] = call_user_func( $map_callback, $match[1], $type );
				} else {
					$blocks[] = $match[1];
				}
			}
		}

		return $blocks;
	}
	public static function parse_block_with_defaults( $block_str, $type ) {
		$attributes = json_decode( $block_str, true );

		// Now insert default values for any missing attributes.
		$fields_settings = \Search_Filter\Fields\Settings::get_settings_by_context( 'block/field/' . $type );
		foreach ( $fields_settings as $field_setting ) {
			if ( ! isset( $attributes[ $field_setting['name'] ] ) && isset( $field_setting['default'] ) ) {
				$attributes[ $field_setting['name'] ] = $field_setting['default'];
			}
		}

		return $attributes;
	}
	public static function parse_block( $block_str, $type ) {
		$attributes = json_decode( $block_str, true );
		return $attributes;
	}
}
