<?php
/**
 * Figures out, based on saved S&F Queries, which WP Queries to affect, by assigning `sf_query_id` to the
 * appropriate WP Queries
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Query\Handler;

use Search_Filter\Fields;
use Search_Filter\Fields\Field;
use Search_Filter\Fields\Field_Factory;
use Search_Filter\Queries;
use Search_Filter\Queries\Query;
use Search_Filter\Settings;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Figures out which query to "Select" or attach to
 */
class Wp {

	/**
	 * Stores a local copy of the WP_Query object.
	 *
	 * @var WP_Query
	 */
	private $wp_query;

	/**
	 * Constructor.
	 *
	 * @param WP_Query $query The instantiated WP_Query object.
	 */
	public function __construct( $query ) {
		$this->wp_query = $query;
	}

	/**
	 * Using the query object, build the WP_Query args.
	 */
	public function get_query_args() {
		$query_args = array();
		// Apply the queries.
		if ( ! empty( $this->wp_query->get( 'search_filter_queries' ) ) ) {

			// TODO - add an option to define if we want to inherit the query or not.
			$query_args = array();
			// $query_args = $this->wp_query->query_vars;

			// The return the fields for the query.
			$queries = $this->wp_query->get( 'search_filter_queries' );
			// TODO - throw an error if there is more thank one query?
			foreach ( $queries as $query ) {
				$query_args = $query->apply_wp_query_args( $query_args );
				$query_args = $this->apply_field_wp_query_args( $query_args, $query->get_fields() );

				// Add filter to customise the query args further.
				// TODO - the naming of this does not match convention.
				$query_args = apply_filters( 'search-filter/query/query_args', $query_args, $query );
			}
		}
		return $query_args;
	}

	public function apply_field_wp_query_args( $query_args, $fields ) {
		foreach ( $fields as $field ) {
			$query_args = $field->apply_wp_query_args( $query_args );
		}
		return $query_args;
	}
}
